--[[
Plugin that shows popup messages
]]

----------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("MessageFrame", "LibSink-2.0")
if not plugin then return end

----------------------------------
-- Locals 

local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0-Plugins")
local LSM = LibStub("LibSharedMedia-3.0")

local nrSlots = 5

local db

----------------------------------
-- Init

function plugin:OnRegister()
	local defaults = {
		profile = {
			PluginEnabled = true,
			GrowUp = true,
			Font = "accid",
			FontSize = 23,
			FadeOutTime = 10,
			SlotHeight = 30,
			sink20OutputSink = "RW",
			ScaleUpTime = 0.3,
			ScaleDownTime = 0.6,
			DefaultSound = "RW: Bell",
			PlaySound = true,
		}
	}
	self.db = RW.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	-- Register for LibSink for message output selection
	self:SetSinkStorage(self.db.profile)
	self:RegisterSink("RW", "Raid Watch", "Messages trough RW", "AddMessage")
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
end

function plugin:PluginEnable()
	if not db.PluginEnabled then
		self:Disable()
		return
	end
	self:RegisterMessage("RW_ConfigMode")
	self:RegisterMessage("RW_Message")
	self:RegisterMessage("RW_DefaultsChanged")
end

function plugin:PluginDisable()
end

----------------------------------
-- Anchor

local function dragStart(self)
	self:StartMoving()
end
local function dragStop(self)
	self:StopMovingOrSizing()
	local s = self:GetEffectiveScale()
	db.x = self:GetLeft() * s
	db.y = self:GetBottom() * s
end

local function onUpdate(self, elapsed)
	local showing = false
	for i, s in next, plugin.slots do
		if s:IsShown() then
			showing = true
			s.ScaleTime = s.ScaleTime + elapsed
			-- Scale the text up untill ScaleUpTime is reached
			if s.ScaleTime < db.ScaleUpTime then
				s:SetScale(s:GetScale() + elapsed * 3)
			-- Scale down untill ScaleDownTime is reached
			elseif s.ScaleTime <= db.ScaleDownTime then
				local newScale = s:GetScale() - elapsed * 3
				s:SetScale(newScale > 0 and newScale or 0.01)
			-- Else reset the scale and text size
			else
				s:SetScale(1)
			end
			FadingFrame_OnUpdate(s)
		else
		end
	end
	if not showing then self:SetScript("OnUpdate", nil) end
end
	
function plugin:CreateAnchor()		
	self.anchor = RW:CreateAnchor("MessageAnchor", UIParent, "Messages")
	local s = self.anchor:GetEffectiveScale()
	self.anchor:SetScript("OnDragStart", dragStart)
	self.anchor:SetScript("OnDragStop", dragStop)
	if db.x then
		self.anchor:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", db.x / s, db.y / s)
	else
		self.anchor:SetPoint("TOP", RaidWarningFrame, "BOTTOM", 0, -50)
	end
	
	local frame = CreateFrame("Frame", nil, UIParent)	
	self.eventFrame = frame
end

function plugin:Reset(var1, var2, var3)
	db = self.db.profile
	self:SetSinkStorage(self.db.profile)
	if self.anchor then
		local s = self.anchor:GetEffectiveScale()
		self.anchor:ClearAllPoints()
		if db.x then
			self.anchor:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", db.x / s, db.y / s)
		else
			self.anchor:SetPoint("TOP", RaidWarningFrame, "BOTTOM", 0, -50)
		end
	end
	if not db.PluginEnabled and self:IsEnabled() then self:Disable() 
	elseif not db.PluginEnabledend and not self:IsEnabled() then self:Enable() end
end

----------------------------------
-- Frame handling

-- Creates and returns one message slot
function plugin:GetMessageFrame()
	if not self.anchor then self:CreateAnchor() end
	
	-- Create message slots
	if not self.slots then
		self.slots = {}
		for i = 1, nrSlots do
			local frame = CreateFrame("Frame", "SlotFrame_"..i, self.eventFrame)
			frame:SetWidth(1)
			frame:SetHeight(db.SlotHeight)
			local str = frame:CreateFontString("RWMessageSlot_"..i, "ARTWORK")
			str:SetPoint("CENTER")
			str:SetFont(LSM:Fetch("font", db.Font), db.FontSize, "OUTLINE")
			
			local icon = frame:CreateTexture()
			icon:SetPoint("RIGHT", str, "LEFT", -4, 0)
			icon:SetWidth(db.SlotHeight - 4)
			icon:SetHeight(db.SlotHeight - 4)
			
			FadingFrame_SetFadeInTime(frame, 0.4)
			FadingFrame_SetFadeOutTime(frame, 3)
			
			frame.text = str
			frame.icon = icon
			
			frame:Hide()
			self.slots[i] = frame
		end
	end

	-- Re-arrange slot, so that slot 1 is always the next one to be used (n:th slot becomes slot nr 1)
	local slots = self.slots
	local old = slots[#slots]
	for i = #slots, 1, -1 do
		if i > 1 then
			slots[i] = slots[i-1]
		else
			slots[i] = old
		end
	end
	
	-- Position the slots according to the slots order visually
	for i, t in pairs(self.slots) do
		t:ClearAllPoints()
		if i == 1 then
			t:SetPoint(db.GrowUp and "BOTTOM" or "TOP", self.anchor, db.GrowUp and "TOP" or "BOTTOM", 0, db.GrowUp and 2 or -2)
		else
			t:SetPoint(db.GrowUp and "BOTTOM" or "TOP", self.slots[i-1], db.GrowUp and "TOP" or "BOTTOM", 0, db.GrowUp and 2 or -2)
		end
		FadingFrame_SetHoldTime(t, db.FadeOutTime)
	end	

	-- Return the "oldest" slot to be re-used
	return self.slots[1]
end

-- Internal function that is called from LibSink
function plugin:AddMessage(addon, text, r, g, b, font, size, outline, sticky, loc, icon)
	local slot = self:GetMessageFrame()
	slot.text:SetText(text)
	slot.text:SetTextColor(r, g, b)
	slot.icon:SetTexture(icon)
	slot.ScaleTime = 0
	FadingFrame_Show(slot)
	
	if not self.eventFrame:GetScript("OnUpdate") then self.eventFrame:SetScript("OnUpdate", onUpdate) end
end

----------------------------------
-- Message Handling

--- Show a new message as popup
-- @param text The text to be shown
-- @param r Red component to the color
-- @param g Green component to the color
-- @param b Blue component to the color
-- @param a Alpha component to the color
-- @param icon Optional icon to show
function plugin:RW_Message(message, text, r, g, b, a, icon, sound)
	if db.PlaySound then
		PlaySoundFile(LSM:Fetch("sound", sound or db.DefaultSound))
	end
	self:Pour(text or "Test message", r, g, b, nil, nil, nil, nil, nil, icon)
end

-- Enter configuration mode
function plugin:RW_ConfigMode(message, start)
	local function randomMessage()
		local spellName, rank, icon = RW:GetRandomSpell()
		plugin:RW_Message("RW_Message", spellName ..(rank ~= "" and (" ("..rank..")" ) or "" ), math.random(), math.random(), math.random(), math.random(), icon)
	end
	if not self.anchor then self:CreateAnchor() end
	if start then
		self.anchor:Show()
		self:ScheduleRepeatingTimer(randomMessage, 2)
	else
		self.anchor:Hide()
		self:CancelAllTimers()
	end
end

function plugin:RW_DefaultsChanged(message, option, value)
	if option == "Font" then
		self.db.profile.Font = value
	end
	if self.slots then
		for i, slot in pairs(self.slots) do
			slot.text:SetFont(LSM:Fetch("font", self.db.profile.Font), self.db.profile.FontSize, "OUTLINE")
		end
	end
end
---------------------------------------------

function plugin:GetOptions()
	self.category = "Alert Plugins"
	local icon = [[Interface\Icons\INV_MISC_NOTE_02]]
	local options = {
		name = "Messages",
		type = "group",
		icon = icon,
		set = function(info, value)
			db[info[#info]] = value
		end,
		get = function(info)
			return db[info[#info]]
		end,
		args = {
			image = {
				type = "description",
				name = L.MESSAGE_FRAME_DESC,
				image = icon,
				width = "full",
				imageWidth = 50,
				imageHeight = 50,
				imageCoords = {0.1,0.9,0.1,0.9},
				order = 1,
			},
			enabled = {
				type = "toggle",
				name = "Enabled",
				set = function(info, value) 
					db.PluginEnabled = value
					if not db.PluginEnabled then
						self:Disable()
					else
						self:Enable()
					end
				end,
				get = function(info) return db.PluginEnabled end,
				order = 2,
			},
			headingBreak = {
				type = "header",
				name = "",
				order = 3,
			},
			TargetFrame = plugin:GetSinkAce3OptionsDataTable(),
			FadeOutTime = {
				type = "range",
				name = L.FADE_OUT_TIME,
				desc = L.FADE_OUT_TIME_DESC,
				step = 1,
				max = 15,
				min = 4,
			},
			GrowUp = {
				type = "toggle",
				name = L.GROW_UP,
				desc = L.GROW_UP_DESC,
			},
			DefaultSound = {
				type = "select",
				name = "Default sound",
				dialogControl = "LSM30_Sound",
				values = AceGUIWidgetLSMlists.sound,
			},
			Reset = {
				type = "execute",
				name = L.RESET_POSITION,
				desc = L.RESET_POSITION_DESC,
				func = function() self.db:ResetProfile() end,
			},
			Test = {
				type = "execute",
				name = "Test",
				func = function() 
					local spellName, rank, icon = RW:GetRandomSpell()
					plugin:SendMessage("RW_Message", spellName ..(rank ~= "" and (" ("..rank..")" ) or "" ), math.random(), math.random(), math.random(), math.random(), icon)
				end,
			},
		}
	}
	
	options.args.TargetFrame.guiInline = true
	
	return options
end

